"""
PyBinder decorators module.
"""

from .providers import Factory, Singleton
from .requires import Requires, RequiresProvider


def provides(key, provider=Factory):
    """
    Provides decorator.

    :param key:
    :param provider:
    :return:
    """
    def decorator(callback):
        callback.key = key
        callback.provider = provider
        return callback
    return decorator


def factory(callback):
    """
    Provider callback decorator.

    :param callback:
    :return:
    """
    callback.provider = Factory
    return callback


def singleton(callback):
    """
    Singleton callback decorator.

    :param callback:
    :return:
    """
    callback.provider = Singleton
    return callback


def requires(instance_of, with_name=None, from_namespace=None):
    """
    Requires dependency instance decorator.

    :param instance_of:
    :param with_name:
    :param from_namespace:
    :return:
    """
    return _requires_decorator(Requires(instance_of, with_name,
                                        from_namespace))


def requires_provider(provider, with_name=None, from_namespace=None):
    """
    Requires dependency provider decorator.

    :param provider:
    :param with_name:
    :param from_namespace:
    :return:
    """
    return _requires_decorator(RequiresProvider(provider, with_name,
                                                from_namespace))


def _requires_decorator(provider):
    """
    Requires dependency decorator factory.

    :param provider:
    :return:
    """
    def decorator(callback):
        if not hasattr(callback, 'dependencies'):
            callback.dependencies = list()
        callback.dependencies.append(provider)
        return callback
    return decorator
