"""
PyBinder exceptions module.
"""


class DiError(Exception):
    """
    Dependency injector error.
    """


class DuplicateKeysError(DiError):
    """
    Dependency injector provider keys duplication error.
    """


class UndefinedProviderError(DiError):
    """
    Dependency injector undefined provider error.
    """


class UndefinedDependencyError(DiError):
    """
    Dependency injector undefined dependency error.
    """


class CircularDependencyError(DiError):
    """
    Dependency injector circular dependency error.
    """


class FactoryError(DiError, TypeError):
    """
    Dependency injector factory error.
    """

    def __init__(self, original_error, context_args, context_kwargs,
                 injected_kwargs):
        """
        Constructor.

        :param original_error:
        :param context_args:
        :param context_kwargs:
        :param injected_kwargs:
        :return:
        """
        super(FactoryError, self).__init__(original_error)
        self.context_args = context_args
        self.context_kwargs = context_kwargs
        self.injected_kwargs = injected_kwargs

    def __str__(self):
        """
        String representation.

        :return:
        """
        import pprint
        debug_info = pprint.PrettyPrinter().pformat(dict(
            context_args=self.context_args,
            context_kwargs=self.context_kwargs,
            injected_kwargs=self.injected_kwargs
        ))
        return ''.join((super(FactoryError, self).__str__(),
                        '\n\nDebug info:\n', debug_info))
