from pybinder import Container
from pybinder.catalogs import Catalog
from pybinder.decorators import provides, requires
from pybinder.providers import Singleton


# Here are some example classes, A and B. Class A is a singleton, class B is
# not.
class A(object):
    pass


class B(object):
    def __init__(self, a):
        self.a = a


# First of all, you need a Catalog, Master.
# I use Catalog classes for defining objects and their dependencies.
class ExampleCatalog(Catalog):

    # Definition of singleton class A.
    # 'object_a' is used as a key.
    @provides('object_a', provider=Singleton)
    def provide_object_a(self):
        return A()

    # Definition of factory for class B.
    # Class B factory requires instance of class A, that is defined using
    # 'object_a' key.
    @provides('object_b')
    @requires('object_a')
    def provide_object_b(self, object_a):
        return B(object_a)

# Here I create a Container and put a catalog instance to it.
container = Container(ExampleCatalog())

# After the container is created, I need to assemble it.
container.assemble()

# Now I can provide you with class A and B instances.
object_a = container.provide('object_a')
object_b_1 = container.provide('object_b')
object_b_2 = container.provide('object_b')


# Also I can inject dependencies to your callbacks, Master:
@container.inject('object_a')
@container.inject('object_b')
def example_callback(object_a, object_b):
    return object_a, object_b

object_a_from_callback, object_b_3 = example_callback()

# I'll make some asserts, to show it works, Master.
assert object_b_1 is not object_b_2 is not object_b_3
assert object_b_1.a is object_b_2.a is object_b_3.a is object_a
assert object_a is object_a_from_callback
