from pybinder import Container
from pybinder.catalogs import Catalog
from pybinder.decorators import provides
from pybinder.providers import Singleton


class A(object):
    pass


class ExampleCatalog(Catalog):

    # Provider will be called every time container needs class A instance.
    @provides('object_a')
    def provide_object_a(self):
        return A()

    # Provider will be called once, result of call will be memorized.
    @provides('object_a_singleton', provider=Singleton)
    def provide_object_a_singleton(self):
        return A()


# Now, let's make assembled container, Master:
container = Container(ExampleCatalog())
container.assemble()

# And ask container to provide us with a few 'object_a' instances:
object_a_1 = container.provide('object_a')
object_a_2 = container.provide('object_a')

# And few 'object_a_singleton' instances:
object_a_singleton_1 = container.provide('object_a_singleton')
object_a_singleton_2 = container.provide('object_a_singleton')

# Here is an assert, Master:
assert object_a_1 is not object_a_2  # Factory
assert object_a_singleton_1 is object_a_singleton_2  # Singleton
