from pybinder import Container
from pybinder.catalogs import Catalog
from pybinder.decorators import provides, requires
from pybinder.providers import Value, Singleton


import sqlite3


class A(object):
    def __init__(self, database):
        self.database = database


class B(object):
    def __init__(self, database):
        self.database = database


class ExampleCatalog(Catalog):
    """
    PyBinder example catalog.
    """

    @provides('database', provider=Singleton)
    @requires('db_config')
    def provide_database(self, db_config):
        return sqlite3.connect(**db_config)

    @provides('object_a')
    @requires('database')
    def provide_object_a(self, database):
        return A(database)

    @provides('object_b')
    @requires('database')
    def provide_object_b(self, database):
        return B(database)


catalog = ExampleCatalog()
catalog.bind('db_config', Value({'database': 'example.db'}))
container = Container(catalog)
container.assemble()

# You can use `container.provide()` method, Master:
a1 = container.provide('object_a')
b1 = container.provide('object_b')


# Or if you have an callback, that needs dependency, please use
# `container.inject()` decorator, Master:
@container.inject('object_a')
@container.inject('object_b')
def callback1(object_a, object_b):
    print object_a, object_b


callback1()


# Sometimes, Master, you may wish to make dependency injection with a specific
# name. I have `with_name` argument in `container.inject()` for such needs:
@container.inject('object_a', with_name='a')
@container.inject('object_b', with_name='b')
def callback2(a, b):
    print a, b


callback2()


# Sometimes your callbacks need to receive context args, Master.
# I have no problems with this:
@container.inject('object_a')
@container.inject('object_b')
def callback3(context1, context2, object_a, object_b):
    print context1, context2, object_a, object_b
    return context1, context2


context1_original = object()
context2_original = object()

context1, context2 = callback3(context1_original,
                               context2=context2_original)

assert context1_original is context1
assert context2_original is context2
