"""
Testing of injection affected callback.
"""
from pybinder import Container
from pybinder.catalogs import Catalog
from pybinder.decorators import singleton, requires
from pybinder.providers import Value

import sqlite3


# Dependency injection container definition part
class ExampleCatalog(Catalog):
    """
    PyBinder example catalog.
    """

    @singleton
    @requires('db_config')
    def database(self, db_config):
        return sqlite3.connect(**db_config)


catalog = ExampleCatalog()
catalog.bind('db_config', Value({'database': 'example.db'}))
container = Container(catalog)
container.assemble()

# Database initialization
database = container.provide('database')
database.execute('CREATE TABLE IF NOT EXISTS users(email text)')


# Dependency injection affected callback
@container.inject('database')
def does_user_exist(email, database):
    cursor = database.execute('SELECT email FROM users WHERE email=?', [email])
    return cursor.fetchone() is not None


# Unit tests
import unittest2 as unittest
from mock import Mock


class DoesUserExistTests(unittest.TestCase):

    def test_does_user_exist_row_found(self):
        # Creating the mock
        database = Mock(sqlite3.Connection)
        cursor = Mock(sqlite3.Cursor)
        cursor.fetchone.return_value = Mock(sqlite3.Row)
        database.execute.return_value = cursor

        # Making asserts
        self.assertTrue(does_user_exist('test@gmail.com', database=database))
