"""
Testing of injection affected class.
"""
from pybinder import Container
from pybinder.catalogs import Catalog
from pybinder.decorators import singleton, requires
from pybinder.providers import Value

import sqlite3


# Dependency injection container definition part
class ExampleCatalog(Catalog):
    """
    PyBinder example catalog.
    """

    @singleton
    @requires('db_config')
    def database(self, db_config):
        return sqlite3.connect(**db_config)


catalog = ExampleCatalog()
catalog.bind('db_config', Value({'database': 'example.db'}))
container = Container(catalog)
container.assemble()

# Database initialization
database = container.provide('database')
database.execute('CREATE TABLE IF NOT EXISTS users(email text)')


# Injection affected class
@container.inject('database')
class UserModel(object):

    def __init__(self, database):
        self.database = database

    def exists(self, email):
        cursor = self.database.execute('SELECT email FROM users WHERE email=?',
                                       [email])
        return cursor.fetchone() is not None

# Unit tests
import unittest2 as unittest
from mock import Mock


class UserModelTests(unittest.TestCase):

    def test_exists_row_found(self):
        # Creating the mock
        database = Mock(sqlite3.Connection)
        cursor = Mock(sqlite3.Cursor)
        cursor.fetchone.return_value = Mock(sqlite3.Row)
        database.execute.return_value = cursor

        # Making asserts
        user = UserModel(database=database)
        self.assertTrue(user.exists('test@gmail.com'))
